module mac_rx_model #(
        parameter           FRM_NUM_MAX             =   16'd25          ,   // 一次循环中发送以太网帧的帧数,最后发送一帧PAUSE帧结束
    
        parameter           D_MAC_ADDR_IDLE         =   48'hff99aa112233,   // 6字节目标地址
        parameter           S_MAC_ADDR_IDLE         =   48'habcd37445566,   // 6字节源地址
        parameter           TYPE_IDLE               =   16'h0800        ,   // 2字节类型

        parameter           DATA_START_BYTE_IDLE    =   8'haa           ,   // 数据字段的第1字节数据,复位后的初始值
        parameter           DATA_START_BYTE_INCREASE=   8'hbb           ,   // 计算下一帧的数据字段的第1字节数据的增量

        parameter           DATA_LENGTH_IDLE        =   16'd67          ,   // 数据字段的长度,复位后的初始值
        parameter           DATA_LENGTH_INCREASE    =   16'd33          ,   // 计算下一帧的数据字段的长度的增量
        parameter           DATA_LENGTH_MAX         =   16'd1500        ,   // 以太网帧的数据字段长度最大为1500字节,另外VLAN帧(0x8100)的最大数据字段长度为1504字节
                                                                            // 以太网帧的数据字段长度最小为46字节,但因tx_fcs_fwd_ena禁用,IP会在发送端对数据字段长度小于46字节的进行补齐操作

        parameter           PAUSE_VALUE_IDLE        =   16'd274         ,   // PAUSE帧的2字节暂停时间,复位后的初始值
        parameter           PAUSE_VALUE_INCREASE    =   16'd735         ,   // 计算下一帧的PAUSE帧的暂停时间的增量

        parameter           PAUSE_ADDRESS_IDLE      =   48'haabbcc      ,   // PAUSE帧的6字节源地址,复位后的初始值
        parameter           PAUSE_ADDRESS_INCREASE  =   48'h113355          // 计算下一帧的PAUSE帧的源地址的增量
    )(
        input               rx_mac_clk,
        input               rx_mac_rst_n,
                
    //CONTROL PORT
        input               i_cycle_one_time,           // 执行一次接收循环的控制信号,下降沿有效
        input               i_cycle_all_time,           // 持续执行接收循环的控制信号,高电平有效

        input               i_rx_length_hold,
        input               i_rx_no_pause,              // 不接受Pause帧
        input               i_rx_check_clear,           // 清除检测结果,高有效

        output  reg         rx_running,                 // 执行发送操作

    //IP RX PORT
        input               rx_mac_valid,               // 接收以太网帧的使能
        input       [ 7:0]  rx_mac_data,                // 接收以太网帧的数据
        input               rx_mac_last,                // 接收以太网帧的最后字节
        input               rx_mac_error,               // 接收以太网帧的错误

        input               rx_statistics_valid,        // 接收统计信息的有效
        input       [26:0]  rx_statistics_vector,       // 接收统计信息
        input               rx_pause_req,               // 接收PAUSE帧的请求
        input       [15:0]  rx_pause_val,               // 接收PASUE帧的2字节暂停时间

    //CHECK RESULT
        output  reg         rx_data_error,              // 接收以太网帧的数据错误
        output  reg [ 1:0]  rx_cnt_error,               // 接收以太网帧的统计长度异常 输出长度异常
        output  reg         rx_statistics_error,        // 接收以太网帧的统计信息中异常标识有效
        output  reg         rx_error_all_data,          // 接收以太网帧异常


        output  reg         rx_data_error_pause,        // 接收PAUSE帧的数据错误
        output  reg [ 1:0]  rx_length_error_pause,      // 接收PAUSE帧的统计长度异常 输出长度异常
        output  reg         rx_statistics_error_pause,  // 接收以太网帧的统计信息中异常标识有效
        output  reg         rx_pause_error,             // 接收PAUSE帧的2字节暂停时间错误
        output  reg         rx_error_all_pause,         // 接收Pause帧异常

        output  reg         rx_error_all,               // RX端接收的任何数据出现异常

        output  reg         rx_error_all_data_check,    // 检测到接收以太网帧异常后常为高
        output  reg         rx_error_all_pause_check,   // 检测到接收Pause帧异常后常为高
        output  reg         rx_error_all_check,         // 检测到RX端接收的任何数据出现异常后常为高

        output  reg         rx_cnt_er,
        output  reg [31:0]  rx_cnt
    );

//**the MAC layer data receiving**//
localparam  IDLE                    =   3'b000;   // 初始状态
localparam  LATCH_DATA              =   3'b001;   // 缓存接收一帧以太网帧需要的信息
localparam  RX_DATA                 =   3'b010;   // 接收一帧以太网帧,并检测接收数据等信息是否正确
localparam  UPDATE_DATA             =   3'b011;   // 计算下一帧以太网帧需要的信息,并记录接收过以太网帧的帧数
localparam  RX_FRM_NUM              =   3'b100;   // 判断是否接收足够帧数的以太网帧
localparam  RX_PAUSE                =   3'b101;   // 接收一帧PAUSE帧,并检测接收数据等信息是否正确
localparam  UPDATE_PAUSE            =   3'b110;   // 计算下一帧PAUSE帧需要的信息

localparam  PRE_DATA_LENGTH_IDLE    =   16'd14;   // 不可修改,固定为6字节目的地址+6字节源地址+2字节类型

reg             rx_mac_valid_d1;
reg     [ 7:0]  rx_mac_data_d1;             /* synthesis syn_keep=1 */
reg             rx_mac_last_d1;
reg             rx_mac_error_d1;
reg             rx_statistics_valid_d1;
reg     [26:0]  rx_statistics_vector_d1;    /* synthesis syn_keep=1 */
reg             rx_pause_req_d1;
reg     [15:0]  rx_pause_val_d1;            /* synthesis syn_keep=1 */

reg             rx_mac_valid_d2;
reg     [ 7:0]  rx_mac_data_d2;             /* synthesis syn_keep=1 */
reg             rx_mac_last_d2;
reg             rx_mac_error_d2;
reg             rx_statistics_valid_d2;
reg     [26:0]  rx_statistics_vector_d2;    /* synthesis syn_keep=1 */
reg             rx_pause_req_d2;
reg     [15:0]  rx_pause_val_d2;            /* synthesis syn_keep=1 */

reg             rx_mac_valid_d3;
reg     [ 7:0]  rx_mac_data_d3;             /* synthesis syn_keep=1 */
reg             rx_mac_last_d3;
reg             rx_mac_error_d3;
reg             rx_statistics_valid_d3;
reg     [26:0]  rx_statistics_vector_d3;    /* synthesis syn_keep=1 */
reg             rx_pause_req_d3;
reg     [15:0]  rx_pause_val_d3;            /* synthesis syn_keep=1 */

reg             rx_mac_valid_d4;
reg     [ 7:0]  rx_mac_data_d4;             /* synthesis syn_keep=1 */
reg             rx_mac_last_d4;
reg             rx_mac_error_d4;
reg             rx_statistics_valid_d4;
reg     [26:0]  rx_statistics_vector_d4;    /* synthesis syn_keep=1 */
reg             rx_pause_req_d4;
reg     [15:0]  rx_pause_val_d4;            /* synthesis syn_keep=1 */

//state machine
reg     [ 2:0]  cur_state;
reg     [ 2:0]  next_state;

reg             cycle_one_time_d1;
reg             cycle_one_time_d2;
reg             cycle_start;

reg             latch_data_end;
reg             rx_data_state;
reg             update_data_end;
reg     [15:0]  rx_frm_cnt;                 /* synthesis syn_keep=1 */
reg             rx_frm_end;
reg             rx_pause_state;

reg     [ 7:0]  data_start_byte;
reg     [15:0]  data_length_next;           /* synthesis syn_keep=1 */
reg     [15:0]  data_length_next_rea;       /* synthesis syn_keep=1 */
reg     [15:0]  data_length;                /* synthesis syn_keep=1 */
reg     [15:0]  frm_length;                 /* synthesis syn_keep=1 */
reg     [15:0]  frm_length_last;            /* synthesis syn_keep=1 */
reg     [15:0]  frm_length_add_4;           /* synthesis syn_keep=1 */
reg             frm_length_less_60;
reg     [111:0] shift_reg;                  /* synthesis syn_keep=1 */
wire    [ 7:0]  pre_data;                   /* synthesis syn_keep=1 */
reg     [ 7:0]  data_increase;              /* synthesis syn_keep=1 */

reg     [15:0]  rx_data_cnt;                /* synthesis syn_keep=1 */
reg             rx_data_cnt_data;
reg             rx_data_cnt_last;
reg             rx_data_cnt_more;
reg             rx_pre_data_check;
reg             rx_data_check;
reg             rx_data_state_d1;
reg             frm_length_less_60_d1;
reg             rx_data_cnt_data_d1;
reg             rx_data_cnt_last_d1;
reg             rx_data_cnt_more_d1;

reg             rx_statistics_length_check_64;
reg             rx_statistics_length_check_frm_a4;
reg             rx_length_check_60;
reg             rx_length_check_frm;

reg     [15:0]  pause_value;            /* synthesis syn_keep=1 */
reg     [47:0]  pause_address;          /* synthesis syn_keep=1 */
reg     [47:0]  pause_data_1;           /* synthesis syn_keep=1 */
reg     [47:0]  pause_data_2;           /* synthesis syn_keep=1 */
reg     [47:0]  pause_data_3;           /* synthesis syn_keep=1 */
reg     [143:0] shift_reg_pause;        /* synthesis syn_keep=1 */
wire    [ 8:0]  pre_data_pause;         /* synthesis syn_keep=1 */
reg     [15:0]  rx_data_cnt_pause;      /* synthesis syn_keep=1 */

reg             rx_pause_state_d1;
reg             rx_data_pause_check;
reg             rx_length_pause_check_60;
reg             rx_data_cnt_pause_less_60;
reg             rx_statistics_pause_length_check_64;
reg             rx_pause_val_check;

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_mac_valid_d1         <=  1'b0;
        rx_mac_data_d1          <=  8'd0;
        rx_mac_last_d1          <=  1'b0;
        rx_mac_error_d1         <=  1'b0;
        rx_statistics_valid_d1  <=  1'b0;
        rx_statistics_vector_d1 <=  27'd0;
        rx_pause_req_d1         <=  1'b0;
        rx_pause_val_d1         <=  16'd0;

        rx_mac_valid_d2         <=  1'b0;
        rx_mac_data_d2          <=  8'd0;
        rx_mac_last_d2          <=  1'b0;
        rx_mac_error_d2         <=  1'b0;
        rx_statistics_valid_d2  <=  1'b0;
        rx_statistics_vector_d2 <=  27'd0;
        rx_pause_req_d2         <=  1'b0;
        rx_pause_val_d2         <=  16'd0;

        rx_mac_valid_d3         <=  1'b0;
        rx_mac_data_d3          <=  8'd0;
        rx_mac_last_d3          <=  1'b0;
        rx_mac_error_d3         <=  1'b0;
        rx_statistics_valid_d3  <=  1'b0;
        rx_statistics_vector_d3 <=  27'd0;
        rx_pause_req_d3         <=  1'b0;
        rx_pause_val_d3         <=  16'd0;

        rx_mac_valid_d4         <=  1'b0;
        rx_mac_data_d4          <=  8'd0;
        rx_mac_last_d4          <=  1'b0;
        rx_mac_error_d4         <=  1'b0;
        rx_statistics_valid_d4  <=  1'b0;
        rx_statistics_vector_d4 <=  27'd0;
        rx_pause_req_d4         <=  1'b0;
        rx_pause_val_d4         <=  16'd0;
    end else begin
        rx_mac_valid_d1         <=  rx_mac_valid;
        rx_mac_data_d1          <=  rx_mac_data;
        rx_mac_last_d1          <=  rx_mac_last;
        rx_mac_error_d1         <=  rx_mac_error;
        rx_statistics_valid_d1  <=  rx_statistics_valid;
        rx_statistics_vector_d1 <=  rx_statistics_vector;
        rx_pause_req_d1         <=  rx_pause_req;
        rx_pause_val_d1         <=  rx_pause_val;

        rx_mac_valid_d2         <=  rx_mac_valid_d1;
        rx_mac_data_d2          <=  rx_mac_data_d1;
        rx_mac_last_d2          <=  rx_mac_last_d1;
        rx_mac_error_d2         <=  rx_mac_error_d1;
        rx_statistics_valid_d2  <=  rx_statistics_valid_d1;
        rx_statistics_vector_d2 <=  rx_statistics_vector_d1;
        rx_pause_req_d2         <=  rx_pause_req_d1;
        rx_pause_val_d2         <=  rx_pause_val_d1;

        rx_mac_valid_d3         <=  rx_mac_valid_d2;
        rx_mac_data_d3          <=  rx_mac_data_d2;
        rx_mac_last_d3          <=  rx_mac_last_d2;
        rx_mac_error_d3         <=  rx_mac_error_d2;
        rx_statistics_valid_d3  <=  rx_statistics_valid_d2;
        rx_statistics_vector_d3 <=  rx_statistics_vector_d2;
        rx_pause_req_d3         <=  rx_pause_req_d2;
        rx_pause_val_d3         <=  rx_pause_val_d2;

        rx_mac_valid_d4         <=  rx_mac_valid_d3;
        rx_mac_data_d4          <=  rx_mac_data_d3;
        rx_mac_last_d4          <=  rx_mac_last_d3;
        rx_mac_error_d4         <=  rx_mac_error_d3;
        rx_statistics_valid_d4  <=  rx_statistics_valid_d3;
        rx_statistics_vector_d4 <=  rx_statistics_vector_d3;
        rx_pause_req_d4         <=  rx_pause_req_d3;
        rx_pause_val_d4         <=  rx_pause_val_d3;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        cur_state   <=  IDLE;
    end else begin
        cur_state   <=  next_state;
    end
end

always @(*) begin
    case(cur_state)
        IDLE        :begin
                        if(cycle_start) begin
                            next_state  =   LATCH_DATA;
                        end else begin
                            next_state  =   IDLE;
                        end
                    end
        LATCH_DATA  :begin
                        if(latch_data_end) begin
                            next_state  =   RX_DATA;
                        end else begin
                            next_state  =   LATCH_DATA;
                        end
                    end
        RX_DATA     :begin
                        if(rx_statistics_valid_d1) begin
                            next_state  =   UPDATE_DATA;
                        end else begin
                            next_state  =   RX_DATA;
                        end
                    end
        UPDATE_DATA :begin
                        if(update_data_end) begin
                            next_state  =   RX_FRM_NUM;
                        end else begin
                            next_state  =   UPDATE_DATA;
                        end
                    end
        RX_FRM_NUM  :begin
                        if(rx_frm_end) begin
                            if(i_rx_no_pause) begin
                                next_state  =   IDLE;
                            end else begin
                                next_state  =   RX_PAUSE;
                            end
                        end else begin
                            next_state  =  LATCH_DATA;
                        end
                    end
        RX_PAUSE    :begin
                        if(rx_statistics_valid_d3) begin
                            next_state  =   UPDATE_PAUSE;
                        end else begin
                            next_state  =   RX_PAUSE;
                        end
                    end
        UPDATE_PAUSE:   next_state  =   IDLE;
        default     :   next_state  =   IDLE;
    endcase
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_running  <=  1'b0;
    end else if(cur_state != IDLE) begin
        rx_running  <=  1'b1;
    end else begin
        rx_running  <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        cycle_one_time_d1   <=  1'b0;
        cycle_one_time_d2   <=  1'b0;
    end else begin
        cycle_one_time_d1   <=  i_cycle_one_time;
        cycle_one_time_d2   <=  cycle_one_time_d1;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        cycle_start <=  1'b0;
    end else if(i_cycle_all_time)begin
        cycle_start <=  1'b1;
    end else if(~cycle_one_time_d1 && cycle_one_time_d2) begin
        cycle_start <=  1'b1;
    end else begin
        cycle_start <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        latch_data_end  <=  1'd0;
    end else if(cur_state == LATCH_DATA)begin
        latch_data_end  <=  ~latch_data_end;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_state   <=  1'd0;
    end else if(latch_data_end)begin
        rx_data_state   <=  1'd1;
    end else if(rx_statistics_valid_d1) begin
        rx_data_state   <=  1'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        update_data_end  <=  1'd0;
    end else if(cur_state == UPDATE_DATA)begin
        update_data_end  <=  ~update_data_end;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_frm_cnt  <=  16'd0;
    end else begin
        case(cur_state)
            IDLE        :   rx_frm_cnt  <=  16'd0;
            UPDATE_DATA :begin
                            if(~update_data_end) begin
                                rx_frm_cnt  <=  rx_frm_cnt + 16'd1;
                            end
                        end
            default     :   rx_frm_cnt  <=  rx_frm_cnt;
        endcase
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_frm_end  <=  1'd0;
    end else if(update_data_end)begin
        if(rx_frm_cnt == FRM_NUM_MAX) begin
            rx_frm_end  <=  1'd1;
        end
    end else begin
        rx_frm_end  <=  1'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_pause_state  <=  1'd0;
    end else if(rx_frm_end && ~i_rx_no_pause)begin
        rx_pause_state  <=  1'd1;
    end else if(rx_statistics_valid_d3) begin
        rx_pause_state  <=  1'd0;
    end
end

//**MAC layer receive the normal data**//
always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        data_start_byte <=  DATA_START_BYTE_IDLE;
    end else if(update_data_end) begin
        data_start_byte <=  data_start_byte + DATA_START_BYTE_INCREASE;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        data_length_next    <=  16'd0;
    end else begin
        data_length_next    <=  data_length + DATA_LENGTH_INCREASE;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        data_length_next_rea    <=  16'd0;
    end else if(data_length_next <= DATA_LENGTH_MAX) begin
        data_length_next_rea    <=  data_length_next;
    end else begin
        data_length_next_rea    <=  data_length_next - DATA_LENGTH_MAX;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        data_length <=  DATA_LENGTH_IDLE;
    end else if(i_rx_length_hold) begin
        data_length <=  DATA_LENGTH_MAX;
    end else if(update_data_end) begin
        data_length <=  data_length_next_rea;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        frm_length  <=  16'd0;
    end else begin
        frm_length  <=  data_length + PRE_DATA_LENGTH_IDLE;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        frm_length_last <=  16'd0;
    end else begin
        frm_length_last <=  frm_length - 16'd2;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        frm_length_add_4    <=  16'd0;
    end else begin
        frm_length_add_4    <=  frm_length + 16'd4;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        frm_length_less_60  <=  1'd0;
    end else if(frm_length < 16'd60) begin
        frm_length_less_60  <=  1'd1;
    end else begin
        frm_length_less_60  <=  1'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        shift_reg   <=  112'd0;
    end else if(latch_data_end) begin
        shift_reg   <=  {D_MAC_ADDR_IDLE,S_MAC_ADDR_IDLE,TYPE_IDLE};
    end else if(rx_mac_valid_d1) begin
        shift_reg   <=  {shift_reg[103:0],8'd0};
    end
end

assign  pre_data    =   shift_reg[111:104];

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        data_increase   <=  8'd0;
    end else if(latch_data_end) begin
        data_increase   <=  data_start_byte;
    end else if(rx_data_cnt_data) begin
        if(rx_mac_valid_d1) begin
            data_increase   <=  data_increase + 8'd1;
        end
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_cnt <=  16'd0;
    end else if(rx_data_state) begin
        if(rx_mac_valid_d1) begin
            rx_data_cnt <=  rx_data_cnt + 16'd1;
        end
    end else begin
        rx_data_cnt <=  16'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_cnt_data    <=  1'd0;
    end else if(rx_data_state) begin
        if( (rx_data_cnt == 16'd13) && rx_mac_valid_d1) begin
            rx_data_cnt_data    <=  1'd1;
        end
    end else begin
        rx_data_cnt_data    <=  1'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_cnt_last    <=  1'd0;
    end else if(rx_mac_valid_d1 && rx_data_cnt_last) begin
        rx_data_cnt_last    <=  1'b0;
    end else if( (rx_data_cnt == frm_length_last) && rx_mac_valid_d1) begin
        rx_data_cnt_last    <=  1'b1;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_cnt_more    <=  1'd0;
    end else if(rx_data_state) begin
        if(rx_data_cnt_last && rx_mac_valid_d1) begin
            rx_data_cnt_more    <=  1'd1;
        end
    end else begin
        rx_data_cnt_more    <=  1'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_pre_data_check   <=  1'b0;
    end else if(rx_mac_data_d1 == pre_data) begin
        rx_pre_data_check   <=  1'b1;
    end else begin
        rx_pre_data_check   <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_check   <=  1'b0;
    end else if(rx_mac_data_d1 == data_increase) begin
        rx_data_check   <=  1'b1;
    end else begin
        rx_data_check   <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_state_d1        <=  1'b0;
        frm_length_less_60_d1   <=  1'b0;
        rx_data_cnt_data_d1     <=  1'b0;
        rx_data_cnt_last_d1     <=  1'b0;
        rx_data_cnt_more_d1     <=  1'd0;
    end else begin
        rx_data_state_d1        <=  rx_data_state;
        frm_length_less_60_d1   <=  frm_length_less_60;
        rx_data_cnt_data_d1     <=  rx_data_cnt_data;
        rx_data_cnt_last_d1     <=  rx_data_cnt_last;
        rx_data_cnt_more_d1     <=  rx_data_cnt_more;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_statistics_length_check_64   <=  1'b0;
    end else if(rx_statistics_vector_d1[21:6] == 16'd64) begin
        rx_statistics_length_check_64   <=  1'b1;
    end else begin
        rx_statistics_length_check_64   <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_statistics_length_check_frm_a4   <=  1'b0;
    end else if(rx_statistics_vector_d1[21:6] == frm_length_add_4) begin
        rx_statistics_length_check_frm_a4   <=  1'b1;
    end else begin
        rx_statistics_length_check_frm_a4   <=  1'b0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_length_check_60  <=  1'b0;
    end else if(rx_data_cnt == 16'd60) begin
        rx_length_check_60  <=  1'b1;
    end else begin
        rx_length_check_60  <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_length_check_frm <=  1'b0;
    end else if(rx_data_cnt == frm_length) begin
        rx_length_check_frm <=  1'b1;
    end else begin
        rx_length_check_frm <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_error   <=  1'b0;
    end else if(rx_data_state_d1) begin
        if(rx_mac_valid_d2 && ~rx_data_cnt_more_d1) begin
            if(rx_data_cnt_data_d1) begin
                rx_data_error   <=  ~rx_data_check;
            end else begin
                rx_data_error   <=  ~rx_pre_data_check;
            end
        end else begin
            rx_data_error   <=  1'b0;
        end
    end else begin
        rx_data_error   <=  1'b0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_cnt_error    <=  2'd0;
    end else if(rx_data_state_d1) begin
        if(rx_statistics_valid_d2) begin
            if(frm_length_less_60_d1) begin
                rx_cnt_error[0] <=  ~rx_statistics_length_check_64;

                rx_cnt_error[1] <=  ~rx_length_check_60;
            end else begin
                rx_cnt_error[0] <=  ~rx_statistics_length_check_frm_a4;

                rx_cnt_error[1] <=  ~rx_length_check_frm;
            end
        end else begin
            rx_cnt_error    <=  2'd0;
        end
    end else begin
        rx_cnt_error    <=  2'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_statistics_error <=  1'd0;
    end else if(rx_data_state_d1) begin
        if(rx_statistics_valid_d2) begin
            rx_statistics_error <=  | rx_statistics_vector_d2[26:22];
        end else begin
            rx_statistics_error <=  1'd0;
        end
    end else begin
        rx_statistics_error <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error_all_data   <=  1'd0;
    end else begin
        rx_error_all_data   <=  rx_data_error || (|rx_cnt_error) || rx_statistics_error;
    end
end

//**MAC layer receive the pause data**//
always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        pause_value <=  PAUSE_VALUE_IDLE;
    end else if(cur_state == UPDATE_PAUSE) begin
        pause_value <=  pause_value + PAUSE_VALUE_INCREASE;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        pause_address   <=  PAUSE_ADDRESS_IDLE;
    end else if(cur_state == UPDATE_PAUSE) begin
        pause_address   <=  pause_address + PAUSE_ADDRESS_INCREASE;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        pause_data_1    <=  48'd0;
    end else begin
        pause_data_1    <=  48'h0180c2000001;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        pause_data_2    <=  48'd0;
    end else begin
        pause_data_2    <=  {pause_address[7:0],pause_address[15:8],pause_address[23:16],pause_address[31:24],pause_address[39:32],pause_address[47:40]};
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        pause_data_3   <=  48'd0;
    end else begin
        pause_data_3   <=  {16'h8808,16'h0001,pause_value};
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        shift_reg_pause <=  144'd0;
    end else if(rx_pause_state) begin
        if(rx_mac_valid_d3) begin
            shift_reg_pause <=  {shift_reg_pause[135:0],8'd0};
        end
    end else begin
        shift_reg_pause <=  {pause_data_1,pause_data_2,pause_data_3};
    end
end

assign  pre_data_pause  =   shift_reg_pause[143:136];

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_cnt_pause   <=  16'd0;
    end else if(rx_pause_state) begin
        if(rx_mac_valid_d3) begin
            rx_data_cnt_pause   <=  rx_data_cnt_pause + 16'd1;
        end
    end else begin
        rx_data_cnt_pause   <=  16'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_pause_state_d1   <=  1'b0;
    end else begin
        rx_pause_state_d1   <=  rx_pause_state;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_pause_check <=  1'b0;
    end else if(rx_mac_data_d3 == pre_data_pause) begin
        rx_data_pause_check <=  1'b1;
    end else begin
        rx_data_pause_check <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_length_pause_check_60    <=  1'b0;
    end else if(rx_data_cnt_pause == 16'd60) begin
        rx_length_pause_check_60    <=  1'b1;
    end else begin
        rx_length_pause_check_60    <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_cnt_pause_less_60   <=  1'b0;
    end else if(rx_data_cnt_pause < 16'd60) begin
        rx_data_cnt_pause_less_60   <=  1'b1;
    end else begin
        rx_data_cnt_pause_less_60   <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_statistics_pause_length_check_64 <=  1'b0;
    end else if(rx_statistics_vector_d3[21:6] == 16'd64) begin
        rx_statistics_pause_length_check_64 <=  1'b1;
    end else begin
        rx_statistics_pause_length_check_64 <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_pause_val_check  <=  1'b0;
    end else if(rx_pause_val_d3 == pause_value) begin
        rx_pause_val_check  <=  1'b1;
    end else begin
        rx_pause_val_check  <=  1'b0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_error_pause <=  1'b0;
    end else if(rx_pause_state_d1) begin
        if(rx_mac_valid_d4) begin
            rx_data_error_pause <=  rx_data_cnt_pause_less_60 && ~rx_data_pause_check;
        end else begin
            rx_data_error_pause <=  1'b0;
        end
    end else begin
        rx_data_error_pause <=  1'b0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_length_error_pause   <=  2'd0;
    end else if(rx_pause_state_d1) begin
        if(rx_statistics_valid_d4) begin
            rx_length_error_pause[0]    <=  ~rx_statistics_pause_length_check_64;

            rx_length_error_pause[1]    <=  ~rx_length_pause_check_60;
        end else begin
            rx_length_error_pause   <=  2'd0;
        end
    end else begin
        rx_length_error_pause   <=  2'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_statistics_error_pause   <=  1'd0;
    end else if(rx_pause_state_d1) begin
        if(rx_statistics_valid_d4) begin
            rx_statistics_error_pause   <=  | rx_statistics_vector_d4[26:22];
        end else begin
            rx_statistics_error_pause   <=  1'd0;
        end
    end else begin
        rx_statistics_error_pause   <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_pause_error  <=  1'd0;
    end else if(rx_pause_state_d1) begin
        if(rx_pause_req_d4) begin
            rx_pause_error  <=  ~rx_pause_val_check;
        end else begin
            rx_pause_error  <=  1'd0;
        end
    end else begin
        rx_pause_error  <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error_all_pause  <=  1'd0;
    end else begin
        rx_error_all_pause  <=  rx_data_error_pause || (|rx_length_error_pause) || rx_statistics_error_pause || rx_pause_error;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error_all    <=  1'd0;
    end else begin
        rx_error_all    <=  rx_error_all_data || rx_error_all_pause;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error_all_data_check <=  1'b0;
    end else if(i_rx_check_clear) begin
        rx_error_all_data_check <=  1'b0;
    end else if(~rx_error_all_data_check) begin
        if(rx_error_all_data) begin
            rx_error_all_data_check <=  1'b1;
        end
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error_all_pause_check    <=  1'b0;
    end else if(i_rx_check_clear) begin
        rx_error_all_pause_check    <=  1'b0;
    end else if(~rx_error_all_pause_check) begin
        if(rx_error_all_pause) begin
            rx_error_all_pause_check    <=  1'b1;
        end
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error_all_check  <=  1'b0;
    end else if(i_rx_check_clear) begin
        rx_error_all_check  <=  1'b0;
    end else if(~rx_error_all_check) begin
        if(rx_error_all) begin
            rx_error_all_check  <=  1'b1;
        end
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_cnt_er   <=  1'd0;
    end else if(i_rx_check_clear) begin
        rx_cnt_er   <=  1'd0;
    end else begin
        if(rx_statistics_valid_d2 && (rx_cnt == 32'hFF_FF_FF_FF) ) begin
            rx_cnt_er   <=  1'd1;
        end else begin
            rx_cnt_er   <=  rx_cnt_er;
        end
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_cnt  <=  32'd0;
    end else if(i_rx_check_clear) begin
        rx_cnt  <=  32'd0;
    end else begin
        if(rx_statistics_valid_d2) begin
            rx_cnt  <=  rx_cnt + 32'd1;
        end else begin
            rx_cnt  <=  rx_cnt;
        end
    end
end

endmodule