module mac_tx_model #(
        parameter           FRM_NUM_MAX             =   16'd25          ,   // 一次循环中发送以太网帧的帧数,最后发送一帧PAUSE帧结束
    
        parameter           D_MAC_ADDR_IDLE         =   48'hff99aa112233,   // 6字节目标地址
        parameter           S_MAC_ADDR_IDLE         =   48'habcd37445566,   // 6字节源地址
        parameter           TYPE_IDLE               =   16'h0800        ,   // 2字节类型

        parameter           DATA_START_BYTE_IDLE    =   8'haa           ,   // 数据字段的第1字节数据,复位后的初始值
        parameter           DATA_START_BYTE_INCREASE=   8'hbb           ,   // 计算下一帧的数据字段的第1字节数据的增量

        parameter           DATA_LENGTH_IDLE        =   16'd67          ,   // 数据字段的长度,复位后的初始值
        parameter           DATA_LENGTH_INCREASE    =   16'd33          ,   // 计算下一帧的数据字段的长度的增量
        parameter           DATA_LENGTH_MAX         =   16'd1500        ,   // 以太网帧的数据字段长度最大为1500字节,另外VLAN帧(0x8100)的最大数据字段长度为1504字节
                                                                            // 以太网帧的数据字段长度最小为46字节,但因tx_fcs_fwd_ena禁用,IP会在发送端对数据字段长度小于46字节的进行补齐操作

        parameter           PAUSE_VALUE_IDLE        =   16'd274         ,   // PAUSE帧的2字节暂停时间,复位后的初始值
        parameter           PAUSE_VALUE_INCREASE    =   16'd735         ,   // 计算下一帧的PAUSE帧的暂停时间的增量

        parameter           PAUSE_ADDRESS_IDLE      =   48'haabbcc      ,   // PAUSE帧的6字节源地址,复位后的初始值
        parameter           PAUSE_ADDRESS_INCREASE  =   48'h113355          // 计算下一帧的PAUSE帧的源地址的增量
    )(
        input               tx_mac_clk,             //
        input               tx_mac_rst_n,           //
        
    //CONTROL PORT
        input               i_cycle_one_time,       // 执行一次发送循环的控制信号,下降沿有效
        input               i_cycle_all_time,       // 持续执行发送循环的控制信号,高电平有效

        input               i_tx_length_hold,
        input               i_tx_no_pause,          // 不发送Pause帧
        input               i_tx_check_clear,       // 清除检测结果,高有效

        output  reg         tx_running,             // 执行发送操作

    //IP TX PORT
        output  reg         tx_mac_valid,           // 发送以太网帧的使能
        output  reg [ 7:0]  tx_mac_data,            // 发送以太网帧的数据
        output  reg         tx_mac_last,            // 发送以太网帧的最后字节
        output  reg         tx_mac_error,           // 发送以太网帧的错误
        input               tx_mac_ready,           // 发送以太网帧的握手

        output  reg         tx_pause_req,           // 发送PAUSE帧的请求
        output  reg [15:0]  tx_pause_val,           // 发送PASUE帧的2字节暂停时间
        output  reg [47:0]  tx_pause_source_addr,   // 发送PASUE帧的6字节源地址
        input               tx_statistics_valid,    // 发送统计信息的有效

        output  reg         tx_cnt_er,
        output  reg [31:0]  tx_cnt
    );

//**the MAC layer data sending**//
localparam  IDLE                    =   3'b000;   // 初始状态
localparam  LATCH_DATA              =   3'b001;   // 缓存发送一帧以太网帧需要的信息
localparam  TX_DATA                 =   3'b010;   // 发送一帧以太网帧
localparam  UPDATE_DATA             =   3'b011;   // 计算下一帧以太网帧需要的信息,并记录发送过以太网帧的帧数
localparam  TX_FRM_NUM              =   3'b100;   // 判断是否发送足够帧数的以太网帧
localparam  TX_PAUSE                =   3'b101;   // 发送一帧PAUSE帧
localparam  UPDATE_PAUSE            =   3'b110;   // 计算下一帧PAUSE帧需要的信息

localparam  PRE_DATA_LENGTH_IDLE    =   16'd14;   // 不可修改,固定为6字节目的地址+6字节源地址+2字节类型

//state machine
reg     [ 2:0]  cur_state;
reg     [ 2:0]  next_state;

reg             cycle_one_time_d1;
reg             cycle_one_time_d2;
reg             cycle_start;

reg             latch_data_end;
reg             latch_data_end_d1;
reg             tx_data_state;
reg             update_data_end;
reg     [15:0]  tx_frm_cnt;
reg             tx_frm_end;
reg             update_pause_state;

reg     [ 7:0]  data_start_byte;
reg     [15:0]  data_length_next;
reg     [15:0]  data_length_next_rea;
reg     [15:0]  data_length;
reg     [15:0]  frm_length;
reg     [15:0]  frm_length_last;

reg     [111:0] shift_reg;
wire    [ 7:0]  pre_data;
reg     [ 7:0]  data_increase;

reg     [15:0]  tx_data_cnt;
reg             tx_data_cnt_data;
reg             tx_data_cnt_last;
reg             tx_data_cnt_last_d1;

reg     [15:0]  pause_value;
reg     [47:0]  pause_address;

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        cur_state   <=  IDLE;
    end else begin
        cur_state   <=  next_state;
    end
end

always @(*) begin
    case(cur_state)
        IDLE        :begin
                        if(cycle_start) begin
                            next_state  =   LATCH_DATA;
                        end else begin
                            next_state  =   IDLE;
                        end
                    end
        LATCH_DATA  :begin
                        if(latch_data_end) begin
                            next_state  =   TX_DATA;
                        end else begin
                            next_state  =   LATCH_DATA;
                        end
                    end
        TX_DATA     :begin
                        if(tx_data_cnt_last_d1 && tx_mac_ready) begin
                            next_state  =   UPDATE_DATA;
                        end else begin
                            next_state  =   TX_DATA;
                        end
                    end
        UPDATE_DATA :begin
                        if(update_data_end) begin
                            next_state  =   TX_FRM_NUM;
                        end else begin
                            next_state  =   UPDATE_DATA;
                        end
                    end
        TX_FRM_NUM  :begin
                        if(tx_frm_end) begin
                            if(i_tx_no_pause) begin
                                next_state  =   IDLE;
                            end else begin
                                next_state  =   TX_PAUSE;
                            end
                        end else begin
                            next_state  =   LATCH_DATA;
                        end
                    end
        TX_PAUSE    :   next_state  =   UPDATE_PAUSE;
        UPDATE_PAUSE:   next_state  =   IDLE;
        default     :   next_state  =   IDLE;
    endcase
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_running  <=  1'b0;
    end else if(cur_state != IDLE) begin
        tx_running  <=  1'b1;
    end else begin
        tx_running  <=  1'b0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        cycle_one_time_d1       <=  1'b0;
        cycle_one_time_d2       <=  1'b0;
    end else begin
        cycle_one_time_d1       <=  i_cycle_one_time;
        cycle_one_time_d2       <=  cycle_one_time_d1;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        cycle_start <=  1'b0;
    end else if(i_cycle_all_time)begin
        cycle_start <=  1'b1;
    end else if(~cycle_one_time_d1 && cycle_one_time_d2) begin
        cycle_start <=  1'b1;
    end else begin
        cycle_start <=  1'b0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        latch_data_end  <=  1'd0;
    end else if(cur_state == LATCH_DATA)begin
        latch_data_end  <=  ~latch_data_end;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        latch_data_end_d1   <=  1'd0;
    end else begin
        latch_data_end_d1   <=  latch_data_end;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_data_state   <=  1'd0;
    end else if(latch_data_end)begin
        tx_data_state   <=  1'd1;
    end else if(tx_data_cnt_last_d1 && tx_mac_ready)begin
        tx_data_state   <=  1'b0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        update_data_end <=  1'd0;
    end else if(cur_state == UPDATE_DATA)begin
        update_data_end <=  ~update_data_end;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_frm_cnt  <=  16'd0;
    end else begin
        case(cur_state)
        IDLE        :   tx_frm_cnt  <=  16'd0;
        UPDATE_DATA :begin
                        if(~update_data_end) begin
                            tx_frm_cnt  <=  tx_frm_cnt + 16'd1;
                        end
                    end
        default     :   tx_frm_cnt  <=  tx_frm_cnt;
        endcase
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_frm_end  <=  1'd0;
    end else if(update_data_end)begin
        if(tx_frm_cnt == FRM_NUM_MAX) begin
            tx_frm_end  <=  1'd1;
        end
    end else begin
        tx_frm_end  <=  1'd0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        update_pause_state  <=  1'd0;
    end else if(cur_state == TX_PAUSE)begin
        update_pause_state  <=  1'd1;
    end else begin
        update_pause_state  <=  1'd0;
    end
end

//**MAC layer send the normal data**//
always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        data_start_byte <=  DATA_START_BYTE_IDLE;
    end else if(update_data_end) begin
        data_start_byte <=  data_start_byte + DATA_START_BYTE_INCREASE;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        data_length_next    <=  16'd0;
    end else begin
        data_length_next    <=  data_length + DATA_LENGTH_INCREASE;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        data_length_next_rea    <=  16'd0;
    end else if(data_length_next <= DATA_LENGTH_MAX) begin
        data_length_next_rea    <=  data_length_next;
    end else begin
        data_length_next_rea    <=  data_length_next - DATA_LENGTH_MAX;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        data_length <=  DATA_LENGTH_IDLE;
    end else if(i_tx_length_hold) begin
        data_length <=  DATA_LENGTH_MAX;
    end else if(update_data_end) begin
        data_length <=  data_length_next_rea;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        frm_length  <=  16'd0;
    end else begin
        frm_length  <=  data_length + PRE_DATA_LENGTH_IDLE;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        frm_length_last <=  16'd0;
    end else begin
        frm_length_last <=  frm_length - 16'd2;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        shift_reg   <=  112'd0;
    end else if(latch_data_end) begin
        shift_reg   <=  {D_MAC_ADDR_IDLE,S_MAC_ADDR_IDLE,TYPE_IDLE};
    end else if(latch_data_end_d1 || tx_mac_ready) begin
        shift_reg   <=  {shift_reg[103:0],8'd0};
    end
end

assign  pre_data    =   shift_reg[111:104];

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        data_increase   <=  8'd0;
    end else if(latch_data_end) begin
        data_increase   <=  data_start_byte;
    end else if(tx_data_cnt_data && tx_mac_ready) begin
        data_increase   <=  data_increase + 8'd1;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_data_cnt <=  16'd0;
    end else if(tx_data_state) begin
        if(latch_data_end_d1 || tx_mac_ready) begin
            tx_data_cnt <=  tx_data_cnt + 16'd1;
        end
    end else begin
        tx_data_cnt <=  16'd0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_data_cnt_data    <=  1'd0;
    end else if(tx_data_state) begin
        if( (tx_data_cnt == 16'd13) && tx_mac_ready) begin
            tx_data_cnt_data    <=  1'd1;
        end
    end else begin
        tx_data_cnt_data    <=  1'd0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_data_cnt_last    <=  1'd0;
    end else if(tx_mac_ready) begin
        if(tx_data_cnt_last) begin
            tx_data_cnt_last    <=  1'b0;
        end else begin
            if(tx_data_cnt == frm_length_last) begin
                tx_data_cnt_last    <=  1'b1;
            end
        end
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_data_cnt_last_d1 <=  1'd0;
    end else if(tx_mac_ready) begin
        tx_data_cnt_last_d1 <=  tx_data_cnt_last;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_mac_valid    <=  1'b0;
    end else if(latch_data_end_d1) begin
        tx_mac_valid    <=  1'b1;
    end else if(tx_data_cnt_last_d1 && tx_mac_ready) begin
        tx_mac_valid    <=  1'b0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_mac_data <=  16'd0;
    end else if(tx_data_cnt_data && tx_mac_ready) begin
        tx_mac_data <=  data_increase;
    end else if(latch_data_end_d1 || tx_mac_ready) begin
        tx_mac_data <=  pre_data;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_mac_error    <=  1'b0;
    end else begin
        tx_mac_error    <=  1'b0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_mac_last <=  1'b0;
    end else if(tx_mac_ready) begin
        tx_mac_last  <=  tx_data_cnt_last;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_cnt_er   <=  1'd0;
    end else if(i_tx_check_clear) begin
        tx_cnt_er   <=  1'd0;
    end else begin
        if(tx_statistics_valid && (tx_cnt == 32'hFF_FF_FF_FF) ) begin
            tx_cnt_er   <=  1'd1;
        end else begin
            tx_cnt_er   <=  tx_cnt_er;
        end
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_cnt  <=  32'd0;
    end else if(i_tx_check_clear) begin
        tx_cnt  <=  32'd0;
    end else begin
        if(tx_statistics_valid) begin
            tx_cnt  <=  tx_cnt + 32'd1;
        end else begin
            tx_cnt  <=  tx_cnt;
        end
    end
end

//**MAC layer send the pause data**//
always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        pause_value <=  PAUSE_VALUE_IDLE;
    end else if(update_pause_state) begin
        pause_value <=  pause_value + PAUSE_VALUE_INCREASE;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        pause_address   <=  PAUSE_ADDRESS_IDLE;
    end else if(update_pause_state) begin
        pause_address   <=  pause_address + PAUSE_ADDRESS_INCREASE;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_pause_req    <=  1'b0;
    end else if(cur_state == TX_PAUSE) begin
        tx_pause_req    <=  1'b1;
    end else begin
        tx_pause_req    <=  1'b0;
    end 
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_pause_val    <=  16'd0;
    end else if(cur_state == TX_PAUSE) begin
        tx_pause_val    <=  pause_value;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_pause_source_addr    <=  48'd0;
    end else if(cur_state == TX_PAUSE) begin
        tx_pause_source_addr    <=  pause_address;
    end
end

endmodule
